import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


/**This file is a debugger.  psi is the polynomial from the book.
   We factor psi as:

   psi = (1/268536)  (t-1) (t-2) 2^{-s} 3^{-s/2} beta

   We then write

   gamma = beta'' - beta'.  (Derivative is with respect to t.)

   Our main test has to do with checking that gamma is
   positive for s in [6,16] and t in [0,4].  This file
   checks that the data we have saved really does describe gamma.

   We first store the much smaller data for
   the function gamma.  This corresponds to a Mathematica file which the reader can
   check against the definition of gamms.  We then randomly extract power combos and
   check the values against the value of gamma*/


public class Debugger {


    /**This runs the debugger. Our approach is to test random values*/
    
    public static void main() {
	double r1=Math.random();
	double r2=Math.random();
	double s=6*(1-r1)+16*r1;
	double t=4*r2;
	t=.02;
	double x=gammaNumerical(s,t);
	double y=reconstructGamma(s,t);
	double z=x/y;
	System.out.println("-------DEBUG--------");
	System.out.println("s: "+s);
	System.out.println("t: "+t);
	System.out.println("gamma numerical   : "+x);
	System.out.println("gamma reconstruct : "+y);
	System.out.println("--------------------");
    }
    /**end main routine*/


    
    

    /**This computes the value of the function gamma, the subject of Lemma A233.*/
    
    public static double gammaNumerical(double s,double t) {
	long[][] m1=gamma1Data();
	long[][] m2=gamma2Data();
	long[][] m3=gamma3Data();
	double d1=extractValue(m1,s,t);
	double d2=extractValue(m2,s,t);
	double d3=extractValue(m3,s,t);
	double x1=Math.pow(3,s/2);
	double x2=Math.pow(4,s/2);
	double x3=Math.pow(6,s/2);
	double sum=x1*d1+x2*d2+x3*d3;
	return sum;
    }

    public static double extractValue(long[][] m,double s,double t) {
	double sum=0;
	for(int i=0;i<3;++i) {
	    for(int j=0;j<8;++j) {
		sum=sum+m[i][j]*Math.pow(s,i)*Math.pow(t,j);
	    }
	}
	return sum;
    }

    public static long[][] gamma1Data() {
    long[][] m={{-546840, -1800480, 99720, -397440, -234600, -33120,  173880, -22080},
	    {18366, 17112, 80766, 24288, 18630, 11592, 4830, -1104},
	    {0, 0, 0, 0, 0, 0, 0, 0}};
    return m;
    }

    public static long[][] gamma2Data() {
    long[][] m={{-345600, -1576320, -509760, -760320, -448800, -63360, 332640, -42240},
		{-199296, -698784, 75216, -149376, -79960, 5856, 94920, -12992},
		{7104, 8432, 33960, 11968, 9180, 5712, 2380, -544}};
    return m;
    }
	
    public static long[][] gamma3Data() {
    long[][] m={{892440, 3376800, 410040, 1157760, 683400, 96480, -506520, 64320},
		{-73350, -246888, -228942, -165792, -110370, -41688,  27510, -2064},
		{1473, 4092, 10557, 5808, 4455, 2772, 1155, -264}};
    return m;
    }
    /**End of the routines computing gamma*/
    

    

    /**This finds the appropriate interval containing t and then
       uses the routine below to evaluate*/

    public static double reconstructGamma(double s,double t) {
	int v0=(int)(Math.floor(4*t));
	double u=4*t-v0;
	return reconstructGamma(v0,s,u);
    }
    

    /**This loads up the power combos and reconstructs the
       value of gamms from them.  This test that our big data file
       is correct.  Here t is a value between 0 and 1.  We take
       a fairly inaccurate rational approximation for the sake
       of speed.*/
    
    public static double reconstructGamma(int v0,double s,double t) {
	int v=8*v0;
	long p=(long)(Math.pow(2,10)*s);
	long q=(long)(Math.pow(2,10));
	BigRational S=new BigRational(p,q); //this is a rational approximation to he exponent s
      
	PowerCombo[] H=new PowerCombo[8];
	double[] d=new double[8];
	
	for(int i=0;i<8;++i) {
	    H[i]=PowerComboExtract.extractBig(v+i);
	    H[i].L=new BigRational(S);
	    H[i].R=new BigRational(S);
	    BigRational X=H[i].evaluateLower();
	    d[i]=X.toDouble();
	}

	//gets the coefficients
        double[] y=new double[8];
	y[0]=d[0];
	for(int i=1;i<8;++i) y[i]=d[i]-d[i-1];

	double sum=0;
	for(int i=0;i<8;++i) sum=sum+y[i]*Math.pow(t,i);
	sum=sum/4096;
	return sum;
    }


    
}

    

